basic_tmpls = {
    "page":"""\
<html>
  <header>
    {% if data.title %}<title>{{data.title}}</title>{% endif %}
    <link rel="stylesheet" href="style.css">
    <meta charset="UTF-8">
  </header>
  <body>
    {% if data.inner %}
        {{data.inner}}
    {% endif %}
  </body>
</html>
""",
    "box": """\
<div 
  {% if data.id %}id='{{data.id}}'{% endif %}
  class='box {% if data.class %}{{data.class}}{% endif %}'
  {% if data.style %}style='{{data.style}}'{% endif %}>

  {% if data.label %}<div class="label">{{data.label}}</div>{% endif %}
  {% if data.text %}<p>{{data.text}}</p>{% endif %}
  {% if data.inner %}
    {{data.inner}}
  {% endif %}
</div>

""",
"textfield": """\
{% if data.label %}<label for="{{data.id}}">{{data.label}}</label>{% endif %}
<input type="text" 
  {% if data.id %}id='{{data.id}}'{% endif %}
  class='box {% if data.class %}{{data.class}}{% endif %}'
  {% if data.style %}style='{{data.style}}'{% endif %}
  {% if data.text %}placeholder="{{data.text}}"{% endif %}
  {% if data.cols %}size="{{data.cols}}"{% endif %}/>

""",
"textarea": """\
{% if data.label %}<label for="{{data.id}}">{{data.label}}</label>{% endif %}
<textarea
  {% if data.id %}id='{{data.id}}'{% endif %}
  class='box {% if data.class %}{{data.class}}{% endif %}'
  {% if data.style %}style='{{data.style}}'{% endif %}
  {% if data.text %}placeholder="{{data.text}}"{% endif %}
  {% if data.cols %}cols="{{data.cols}}"{% endif %}
  {% if data.rows %}rows="{{data.rows}}"{% endif %} ></textarea>
    
""",
"select": """\
{% if data.label %}<label for="{{data.id}}">{{data.label}}</label>{% endif %}
<select
  {% if data.id %}id='{{data.id}}'{% endif %}
  class='box {% if data.class %}{{data.class}}{% endif %}'
  {% if data.style %}style='{{data.style}}'{% endif %}>
  {% if data.text %}<option value="">{{data.text}}</option>{% endif %}
</select>

""",
"radio": """\
<input type="radio"
  {% if data.id %}id='{{data.id}}'{% endif %}
  class='box {% if data.class %}{{data.class}}{% endif %}'
  {% if data.style %}style='{{data.style}}'{% endif %}
  {% if data.cols %}size="{{data.cols}}"{% endif %}/>
<label for="{{data.id}}">{{data.label}}</label>

""",
"checkbox": """\
<input type="checkbox"
  {% if data.id %}id='{{data.id}}'{% endif %}
  class='box {% if data.class %}{{data.class}}{% endif %}'
  {% if data.style %}style='{{data.style}}'{% endif %} />
<label for="{{data.id}}">{{data.label}}</label>

""",
"button": """\
<button
  {% if data.id %}id='{{data.id}}'{% endif %}
  class='box {% if data.class %}{{data.class}}{% endif %}'
  {% if data.style %}style='{{data.style}}'{% endif %}>
  {% if data.label %}{{data.label}}{% endif %}
</button>

""",
"flex":"""\
<div 
  {% if data.id %}id='{{data.id}}'{% endif %}
  {% if data.class %}class='flex {{data.class}}'{% endif %}
  {% if data.style %}style='display:flex;{{data.style}}'{% endif %}>

  {% if data.label %}<div class="box-label">{{data.label}}</div>{% endif %}
  {% if data.text %}<p>{{data.text}}</p>{% endif %}
  {% if data.inner %}
    {{data.inner}}
  {% endif %}
</div>

""",
    "grid":"""\
<div 
  {% if data.id %}id='{{data.id}}'{% endif %}
  {% if data.class %}class='{{data.class}}'{% endif %}
  {% if data.style %}style='display:grid;width:100%;height:100%;{{data.style}}'{% endif %}>

  {% if data.label %}<div class="box-label">{{data.label}}</div>{% endif %}
  {% if data.text %}<p>{{data.text}}</p>{% endif %}
  {% if data.inner %}
    {{data.inner}}
  {% endif %}
</div>

""",
    "p": """\
<p
  {% if data.id %}id='{{data.id}}'{% endif %}
  {% if data.class %}class='{{data.class}}'{% endif %}
  {% if data.style %}style='{{data.style}}'{% endif %}>

  {% if data.text %}{{data.text}}{% endif %}</p>

""",
}

def grid_size(p):
    # sizeはfullまたはNxNの形式
    # fullの場合は
    if p == "full":
        return "width:100%; height:100%;"
    elif p.startswith("w"):
        # wの場合は、widthを指定する
        width_value = p[1:]  # "w"を除いた部分を取得
        if width_value.isdigit():
            return f"width:{width_value}px; height:auto;"
        else:
            raise ValueError(f"Invalid width value: {width_value}")
    elif "x" in p:
        # NxNの場合は、widthとheightを指定する
        size = p.split("x")
        if len(size) != 2:
            raise ValueError(f"Invalid size format: {p}")
        if not size[0].isdigit() or not size[1].isdigit():
            raise ValueError(f"Invalid size format: {p}")
        # size[0]がwidth、size[1]がheight
        width = size[0]
        height = size[1]
        return f"width:{width}%; height:{height}%;"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid size format: {p}")

def grid_tile(p):
    # tileはNxNの形式
    # NxNの場合は、widthとheightを指定する
    if "x" in p:
        size = p.split("x")
        if len(size) != 2:
            raise ValueError(f"Invalid tile format: {p}")
        if not size[0].isdigit() or not size[1].isdigit():
            raise ValueError(f"Invalid tile format: {p}")
        # size[0]がwidth、size[1]がheight
        # それぞれのサイズをfrで指定する
        width = f"{size[0]}"
        height = f"{size[1]}"
        return f"grid-template-columns: repeat({width}, 1fr); grid-template-rows: repeat({height}, 1fr);"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid tile format: {p}")

def flex_size(p):
    # sizeはfullまたはNxNの形式
    # fullの場合は
    if p == "full":
        return "width:100%; height:100%;"
    elif "x" in p:
        # NxNの場合は、widthとheightを指定する
        size = p.split("x")
        if len(size) != 2:
            raise ValueError(f"Invalid size format: {p}")
        if not size[0].isdigit() or not size[1].isdigit():
            raise ValueError(f"Invalid size format: {p}")
        # size[0]がwidth、size[1]がheight
        # それぞれのサイズをパーセントで指定する
        width = size[0]
        height = size[1]
        return f"width:{width}%; height:{height}%;"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid size format: {p}")

def flex_direction(p):
    # directionはrowまたはcolumnの形式
    if p == "row":
        return "flex-direction: row;"
    elif p == "column":
        return "flex-direction: column;"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid direction format: {p}")
def flex_wrap(p):
    # wrapはwrapまたはnowrapの形式
    if p == "wrap":
        return "flex-wrap: wrap;"
    elif p == "nowrap":
        return "flex-wrap: nowrap;"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid wrap format: {p}")
def flex_align(p):
    # alignはstart、center、endの形式
    if p == "start":
        return "align-items: flex-start;"
    elif p == "center":
        return "align-items: center;"
    elif p == "end":
        return "align-items: flex-end;"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid align format: {p}")
def flex_justify(p):
    # justifyはstart、center、endの形式
    if p == "start":
        return "justify-content: flex-start;"
    elif p == "center":
        return "justify-content: center;"
    elif p == "end":
        return "justify-content: flex-end;"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid justify format: {p}")
def flex_item(p):
    # itemはstart、center、endの形式
    if p == "start":
        return "align-self: flex-start;"
    elif p == "center":
        return "align-self: center;"
    elif p == "end":
        return "align-self: flex-end;"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid item format: {p}")
def flex_order(p):
    # orderは整数の形式
    if p.isdigit():
        return f"order: {p};"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid order format: {p}")
def flex_grow(p):
    # growは整数の形式
    if p.isdigit():
        return f"flex-grow: {p};"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid grow format: {p}")
def flex_shrink(p): 
    # shrinkは整数の形式
    if p.isdigit():
        return f"flex-shrink: {p};"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid shrink format: {p}")
def flex_basis(p):
    # basisは整数の形式
    if p.isdigit():
        return f"flex-basis: {p};"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid basis format: {p}")
def box_gpos(p):
    # gsizeはNxNの形式
    # NxNの場合は、grid-columnとgrid-rowを指定する
    # それ以外の形式はエラー
    if "/" in p:
        size = p.split("/")
        if len(size) != 2:
            raise ValueError(f"Invalid gsize format: {p}")
        # size[0]がgrid-column、size[1]がgrid-row
        x = size[0].split("-")
        y = size[1].split("-")
        # FIXME: width, heightの値が一つしかない場合は、w-h形式。二つある場合はw1/22の形式に変換
        if len(x) == 1:
            width = f"{x[0]}"
        elif len(x) == 2:
            width = f"{x[0]} / {x[1]}"
        else:
            raise ValueError(f"Invalid gsize format: {p}")
        if len(y) == 1:
            height = f"{y[0]}"
        elif len(y) == 2:
            height = f"{y[0]} / {y[1]}"
        else:
            raise ValueError(f"Invalid gsize format: {p}")
        return f"grid-column: {width}; grid-row: {height};"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid gsize format: {p}")
def box_gcol(p):
    # gcolはN-Nの形式
    # N-Nの場合は、grid-columnをN\Nの形で指定する
    # Nのみの指定の場合は、位置だけを指定する
    # それ以外の形式はエラー
    if "/" in p:
        size = p.split("/")
        if len(size) != 2:
            raise ValueError(f"Invalid gsize format: {p}")
        return f"grid-column: {p};"
    elif p.isdigit():
        return f"grid-column: {p};"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid gsize format: {p}")
def box_grow(p):
    # growは整数の形式
    if "/" in p:
        size = p.split("/")
        if len(size) != 2:
            raise ValueError(f"Invalid grow format: {p}")
        # size[0]がwidth、size[1]がheight
        # それぞれのサイズを文字列のまま指定する
        return f"grid-row: {p};"
    elif p.isdigit():
        return f"grit-row: {p};"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid grow format: {p}")
def box_size(p):
    # sizeはNxNの形式
    # NxNの場合は、widthとheightを指定する
    # それぞれのサイズを文字列のまま指定する
    # それ以外の形式はエラー
    if "x" in p:
        size = p.split("x")
        if len(size) != 2:
            raise ValueError(f"Invalid size format: {p}")
        if not size[0].isdigit() or not size[1].isdigit():
            raise ValueError(f"Invalid size format: {p}")
        # size[0]がwidth、size[1]がheight
        width = size[0]
        height = size[1]
        return f"width:{width}; height:{height};"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid size format: {p}")

def box_scroll(p):
    if p == "x":
        return "overflow-x: scroll;"
    elif p == "y":
        return "overflow-y: scroll;"
    elif p == "both":
        return "overflow: scroll;"
    else:
        # それ以外の形式はエラー
        raise ValueError(f"Invalid scroll format: {p}")

basic_methods = {
    "grid_size": grid_size,
    "grid_tile": grid_tile,
    "flex_size": flex_size,
    "flex_direction": flex_direction,
    "flex_wrap": flex_wrap,
    "flex_align": flex_align,
    "flex_justify": flex_justify,
    "flex_item": flex_item,
    "flex_order": flex_order,
    "flex_grow": flex_grow,
    "flex_shrink": flex_shrink,
    "flex_basis": flex_basis,
    "box_size": box_size,
    "box_gpos": box_gpos,
    "box_gcol": box_gcol,
    "box_grow": box_grow,
    "box_scroll": box_scroll,
}


from core.mukurol import MukuroL

text = """\
page text:サンプルページ
    grid size:full tile:10x10
        box label:Header gsize:10x2 text:ここにヘッダーが入ります
        box label:Sidebar gsize:3x7 text:サイドバー。メニューなど
        box label:Body gsize:7x7 text:本文。メインコンテンツ
            flex size:full direction:column
                box label:Form text:ここに入力フォームが入ります
                    フォームは部品をテンプレート化した方が良いかもしれません
                box label:Preview text:入力結果がプレビューされます
            ref greeting
            ref farewell
        box label:Footer gsize:10x1 text:フッターです
"""
# Example usage:
if __name__ == '__main__':
    # Create an instance of the HTMLGenerator
    generator = MukuroL()

    # Generate HTML using the templates and data
    html_output = generator.generate_html(text)
    print(html_output)


#!/usr/bin/env python3
import os
import shutil
import argparse
import yaml
import time
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
from .core.mukurol import MukuroL  # MukuroLクラスのインポート
from bs4 import BeautifulSoup
import threading
import traceback

# デフォルト設定
DEFAULT_SRC_DIR = "src"
DEFAULT_DIST_DIR = "dist"
DEFAULT_CONFIG_FILE = "mkl.config.yml"
DEFAULT_PORT = 6961

class MKLFileHandler(FileSystemEventHandler):
    def __init__(self, src_dir, dist_dir, mukurol):
        self.src_dir = src_dir
        self.dist_dir = dist_dir
        self.mukurol = mukurol

    def on_modified(self, event):
        if event.event_type == 'modified' and event.src_path.endswith(".mkl"):
            print(f"File {event.src_path} has been modified. Generating HTML...")
            self.generate_html(event.src_path)
            print("HTML generated. Refresh your browser.")

    def generate_html(self, src_file):
        # ファイル名から出力ファイル名を決定
        base_name = src_file.replace("src/", "").replace(".mkl", ".html")
        dist_file = os.path.join(self.dist_dir, base_name)
        os.makedirs(os.path.dirname(dist_file), exist_ok=True)
        print(f"Generating HTML from {src_file} to {dist_file}")
       
        try:
            with open(src_file, "r") as f:
                mukurol_text = f.read()
            layout_html = self.mukurol.generate_html(mukurol_text)
            # レイアウトHTMLをファイルに出力
            if layout_html != "":
                with open(dist_file, "w") as f:
                    f.write(str(layout_html))
        except Exception as e:
            print(f"Error generating HTML for {src_file}: {e}")
            traceback.print_exc()

def get_all_files(directory):
    """
    指定されたディレクトリ内のすべてのファイル（サブディレクトリを含む）を取得します。
    """
    file_list = []
    for root, _, files in os.walk(directory):
        for file in files:
            file_path = os.path.join(root, file)
            file_list.append(file_path)
    return file_list

def init_command(path):
    """
    指定のディレクトリに、src, distディレクトリと、空のmkl.config.ymlを作成します。
    """
    src_path = os.path.join(path, DEFAULT_SRC_DIR)
    dist_path = os.path.join(path, DEFAULT_DIST_DIR)
    config_path = os.path.join(path, DEFAULT_CONFIG_FILE)

    os.makedirs(src_path, exist_ok=True)
    os.makedirs(dist_path, exist_ok=True)

    with open(config_path, "w") as f:
        yaml.dump({}, f)  # 空のYAMLファイルを作成

    print(f"Initialized MukuroL project in {path}")
    print(f"  - Created directory: {src_path}")
    print(f"  - Created directory: {dist_path}")
    print(f"  - Created file: {config_path}")

def generate_html_file(src_file, output_file, mukurol):
    """
    指定のソースファイルからHTMLファイルを生成します。
    """
    try:
        # 出力先のディレクトリが存在しない場合は作成
        os.makedirs(os.path.dirname(output_file), exist_ok=True)
        print(f"Generating HTML from {src_file} to {output_file}")

        with open(src_file, "r") as f:
            mukurol_text = f.read()
        layout_html = mukurol.generate_html(mukurol_text)
        if layout_html != "":
            with open(output_file, "w") as f:
                f.write(str(layout_html))
        print(f"Generated HTML from {src_file} to {output_file}")
    except Exception as e:
        print(f"Error generating HTML for {src_file}: {e}")
        traceback.print_exc()

def generate_command(input_file=None, output_file=None):
    """
    指定のソースファイルから、出力先ファイルにHTMLファイルを作成します。
    未指定の場合は、srcディレクトリのファイル全てからdistディレクトリに結果を吐き出します。
    """
    mukurol = MukuroL()  # MukuroLクラスのインスタンスを作成

    if input_file:
        # 単一ファイルの処理
        src_file = input_file
        if not output_file:
            # 出力ファイル名が未指定の場合は、デフォルトのdistディレクトリに出力
            base_name = os.path.basename(src_file).replace(".mkl", ".html")
            output_file = os.path.join(DEFAULT_DIST_DIR, base_name)

        process_single_file(src_file, output_file, mukurol)

    else:
        # srcディレクトリのファイルを全て処理
        process_all_files(DEFAULT_SRC_DIR, DEFAULT_DIST_DIR, mukurol)

def process_single_file(src_file, output_file, mukurol):
    """
    単一のMKLファイルを処理してHTMLを生成します。
    """
    generate_html_file(src_file, output_file, mukurol)

def process_all_files(src_dir, dist_dir, mukurol):
    """
    srcディレクトリ以下のすべてのMKLファイルを処理してHTMLを生成します。
    """
    if not os.path.exists(src_dir):
        print(f"Error: Source directory '{src_dir}' not found.")
        return

    mkl_files = [f.replace("src/", '') for f in get_all_files(src_dir) if f.endswith(".mkl")]

    for filename in mkl_files:
        src_file = os.path.join(src_dir, filename)
        base_name = filename.replace(".mkl", ".html")
        dist_file = os.path.join(dist_dir, base_name)
        os.makedirs(os.path.dirname(dist_file), exist_ok=True)

        process_single_file(src_file, dist_file, mukurol)

def watch_command():
    """
    src内のmklファイルを監視し、更新があった場合は、そのmklファイルからHTMLを生成します。
    """
    src_dir = DEFAULT_SRC_DIR
    dist_dir = DEFAULT_DIST_DIR

    if not os.path.exists(src_dir):
        print(f"Error: Source directory '{src_dir}' not found.")
        return

    os.makedirs(dist_dir, exist_ok=True)

    mukurol = MukuroL()  # MukuroLクラスのインスタンスを作成

    # 初期HTMLファイルを生成
    mkl_files = [f.replace("src/",'') for f in get_all_files(src_dir) if f.endswith(".mkl")]
    for filename in mkl_files:
        print(f"Generating initial HTML from {filename}!!!")
        if filename.endswith(".mkl"):
            src_file = os.path.join(src_dir, filename)
            MKLFileHandler(src_dir, dist_dir, mukurol).generate_html(src_file)

    print("Starting file watcher...")
    # ファイル監視の設定
    event_handler = MKLFileHandler(src_dir, dist_dir, mukurol)
    observer = Observer()
    observer.schedule(event_handler, src_dir, recursive=True)
    observer.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()

def main():
    parser = argparse.ArgumentParser(description="MukuroL CLI tool")
    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    # init コマンド
    init_parser = subparsers.add_parser("init", help="Initialize a new MukuroL project")
    init_parser.add_argument("path", help="Path to the project directory")

    # generate コマンド
    generate_parser = subparsers.add_parser("generate", help="Generate HTML from MKL files")
    generate_parser.add_argument("-i", "--input", dest="input_file", help="Path to the input MKL file")
    generate_parser.add_argument("-o", "--output", dest="output_file", help="Path to the output HTML file")

    # watch コマンド
    subparsers.add_parser("watch", help="Watch for changes in the source directory and automatically regenerate HTML")

    args = parser.parse_args()

    if args.command == "init":
        init_command(args.path)
    elif args.command == "generate":
        generate_command(args.input_file, args.output_file)
    elif args.command == "watch":
        watch_command()
    else:
        parser.print_help()

if __name__ == "__main__":
    main()


"""
テキストを分解し、ツリー構造のNode配列とする。
各行は、トリミングした上で、半角スペースで配列とする
"""
from pprint import pprint
import json
class TreeNode:
    def __init__(self, value):
        self.value = value
        self.children = []

    def __repr__(self):
        return f"TreeNode({self.value}, {self.children})"
    def to_dict(self):
        return {
            "value": self.value,
            "children": [child.to_dict() for child in self.children]  # 再帰的に辞書化
        }

def parse_indented_text(text):
    lines = text.split("\n")
    root = TreeNode("root")
    stack = [(0, root)]  # (indent_level, node)
    current = root
    for line in lines:
        stripped = line.lstrip()
        if not stripped:
            continue  # 空行をスキップ
        indent_level = len(line) - len(stripped)
        elements = stripped.split(" ")

        new_node = TreeNode(elements)
        # インデントレベルに応じて親を決定
        if indent_level == 0:
            current = root
        else :
            while stack and stack[-1][0] >= indent_level:
                stack.pop()
                current = stack[-1][1]
        
        if stack:
            stack.append((indent_level,new_node))
            current.children.append(new_node)
            current = new_node

        stack.append((indent_level, new_node))
    return root


"""
コンバート済みのParsedNodeを用いて、HTMLを生成する。
- tmplの値に対応するテンプレートを用いる
- パラメータをそのまま渡す
- childrenがいる場合は、childrenを再帰的2にレンダリングし、{{inner}}に入れる
"""
from .basic_tmpls import basic_tmpls
from jinja2 import Template

class Renderer:
    def __init__(self, node):
        self.node = node

    def render(self):
        # tmplに対応するテンプレートを取得
        tmpl = basic_tmpls[self.node.tmpl]
        # jinja2のテンプレートを生成
        template = Template(tmpl)
        # paramsをそのまま渡す
        params = self.node.params.copy()
        # childrenがいる場合は、childrenを再帰的にレンダリングし、{{inner}}に入れる
        if self.node.children:
            inner_list = []
            for child in self.node.children:
                rendered_child = Renderer(child).render()
                if isinstance(rendered_child, str):
                    inner_list.append(rendered_child)
                else:
                    # エラー処理：文字列以外の場合はスキップするか、エラーログを出力する
                    print(f"Warning: child render result is not a string: {type(rendered_child)}")
            inner = "".join(inner_list)
            params["inner"] = inner
        
        return template.render({"data":params})


"""
TreeNodeをParsedNodeに変換
- 先頭がテンプレート名と一致するなら、tmplに入れる
- 先頭がテンプレート名と一致しないなら、tmplはpとし行は結合し直してtextとする
- 二つ目以降の値は、
    - ":"区切りなら、左辺をキーとしたdictを作ってParamsに入れる
    - ":"を含まなければ、そのままキーとして、値はTrueとしてParamsに入れる
- 再帰的にchildrenを処理する
"""
from .basic_tmpls import basic_tmpls
class ParsedNode:
    def __init__(self, tmpl, params):
        self.tmpl = tmpl
        self.params = params
        self.children = []

    def __repr__(self):
        return f"ParsedNode(tmpl={self.tmpl}, params={self.params}, children={self.children})"
    
    def to_dict(self):
        return {
            "tmpl": self.tmpl,
            "params": self.params,
            "children": [child.to_dict() for child in self.children]  # 再帰的に辞書化
        }

class parser:
    def __init__(self, tree):
        self.tree = tree

    def parse(self):
        # ノードをParsedNodeに変換
        return self.parse_tree_node(self.tree)

    # TreeNodeをParsedNodeに変換する
    def parse_tree_node(self, node):
        tmpl = None
        params = {}
        # 先頭の要素がテンプレート名と一致するか確認
        if node.value[0] in basic_tmpls.keys():
            # テンプレート名が一致する場合、tmplに入れる
            tmpl = node.value[0]
            # 二つ目以降の要素を処理する
            for param in node.value[1:]:
                if ":" in param:
                    key, value = param.split(":", 1)
                    params[key] = value
                else:
                    params[param] = True
        else:
            # テンプレート名が一致しない場合、tmplはpとし行は結合し直してtextとする
            tmpl = "p"
            params["text"] = " ".join(node.value)
        
        # 子ノードを再帰的に処理する
        parsed_node = ParsedNode(tmpl, params)
        for child in node.children:
            parsed_node.children.append(self.parse_tree_node(child))
        return parsed_node